/*----------------------------------------------------------------------------*-

  ttrd2-08b-t0401a-v001a_processor_task.c (Release 2017-08-22a)

  ----------------------------------------------------------------------------
   
  Processor Task.
  
  For Nucleo-F401RE board.

  Supports Mode changes requested by user (by pressing Button 1)

  Please refer to 'ERES2' (Chapter 8) for further information.

-*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*-

  This code is copyright (c) 2014-2017 SafeTTy Systems Ltd.

  This code forms part of a Time-Triggered Reference Design (TTRD) that is 
  documented in the following book: 

   Pont, M.J. (2016) 
   "The Engineering of Reliable Embedded Systems (Second Edition)", 
   Published by SafeTTy Systems Ltd. ISBN: 978-0-9930355-3-1.

  Both the TTRDs and the above book ("ERES2") describe patented 
  technology and are subject to copyright and other restrictions.

  This code may be used without charge: [i] by universities and colleges on 
  courses for which a degree up to and including 'MSc' level (or equivalent) 
  is awarded; [ii] for non-commercial projects carried out by individuals 
  and hobbyists.

  Any and all other use of this code and / or the patented technology 
  described in ERES2 requires purchase of a ReliabiliTTy Technology Licence:
  http://www.safetty.net/technology/reliabilitty-technology-licences

  Please contact SafeTTy Systems Ltd if you require clarification of these 
  licensing arrangements: http://www.safetty.net/contact

   CorrelaTTor, DuplicaTTor, MoniTTor, PredicTTor, ReliabiliTTy, SafeTTy, 
   SafeTTy Systems and WarranTTor are registered trademarks or trademarks 
   of SafeTTy Systems Ltd in the UK and other countries.

-*----------------------------------------------------------------------------*/

#include "../main/main.h"

// ADC library is used to measure and report CPU temperature
#include "../tasks/ttrd2-05a-t0401a-v001a_adc_task.h"

// Support for UART / USB reporting on Nucleo board
#include "../tasks/ttrd2-05a-t0401a-v001a_uart2_buff_o_task.h"

// Requires switch (Button 1) interface
#include "ttrd2-05a-t0401a-v001a_switch_task.h"

/*----------------------------------------------------------------------------*-

  PROCESSOR_TASK_Init()

  Prepare for Processor Task - see below.
  
  PARAMETERS:
     None.

  LONG-TERM DATA:
     None.

  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void PROCESSOR_TASK_Init(void)
   {
   // Do nothing
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_TASK_Update()

  Periodic task that manages switch-based mode transitions.

  This version is employed in all "normal" modes.

  Call every second in this demo.
   
  PARAMETERS:
     None.

  LONG-TERM DATA:
     None.

  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     Data-integrity check on Seconds_count_s.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     Only PreCs.

  WCET:
     Not yet determined.

  BCET:
     Not yet determined.

  RETURN VALUE:
     RETURN_NORMAL_STATE or RETURN_ABNORMAL_STATE.
 
-*----------------------------------------------------------------------------*/
void PROCESSOR_TASK_Update(void)
   {
   uint32_t Change_mode = 0;
   eProc_MoSt New_mode;

   // Check pre-conditions (START)
   // No PreCs
   // Check pre-conditions (END)

   // Read the switch state
   if (SWITCH_BUTTON1_Get_State() == BUTTON1_PRESSED)
      {
      // Switch pressed - change mode
      Change_mode = 1;
      }

   // Handle mode changes
   if (Change_mode == 1)
      {
      switch (PROCESSOR_Get_MoSt())
         {
         default:
            {
            // Do nothing ...
            break;
            }

         case NORMAL_01_M:
            {
            New_mode = NORMAL_02_M;
            break;
            }

         case NORMAL_02_M:
            {
            New_mode = NORMAL_03_M;
            break;
            }

         case NORMAL_03_M:
            {
            New_mode = NORMAL_01_M;
            break;
            }
         }

      // Change mode (no fault)
      PROCESSOR_Store_Reqd_MoSt(New_mode);
      PROCESSOR_Store_PFC(PFC_NO_FAULT);
      PROCESSOR_Change_MoSt();   
      }
   }

/*----------------------------------------------------------------------------*-
  ------------------------------ END OF FILE ---------------------------------
-*----------------------------------------------------------------------------*/
